(***************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**              Terminal - program demostracyjny filozofii sieci          **
**                      Wersja z dnia : 12.19.2018                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                Podziękowania: Natasza Biecek                           **
**                                                                        **
****************************************************************************
***************************************************************************)

unit terminal_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, ueled, SdpoSerial, Forms, Controls, Graphics,
  Dialogs, StdCtrls, ExtCtrls, LCLType, synaser;


const
  CyclicBufferSize             = 2048 ;

type
  CyclicBufferRecT             = record
                                   FrIndex        : word ;
                                   ToIndex        : word ;
                                   Buffer         : array [ 0 .. CyclicBufferSize - 1 ] of char ;
                                 end (* record *) ;

  { TSerwerCfgForm }

  TSerwerCfgForm = class ( TForm )
    ClearButton: TButton;
                     SendButton          : TButton ;
                     OpenPortButton      : TButton ;
                     ExitButton          : TButton ;
                     ClosePortButton     : TButton ;
                     CfgDataLabel        : TLabel ;
                     PortLabel           : TLabel ;
                     SerialPortComboBox  : TComboBox ;
                     CfgDataEdit         : TEdit ;
                     TextMemo            : TMemo ;
                     SerialChannel       : TSdpoSerial ;
                     ConnectLED          : TuELED ;
                     GeneralTimer        : TTimer ;
                     procedure ClearButtonClick(Sender: TObject);
                     procedure FormCreate ( Sender : TObject ) ;
                     procedure OpenPortButtonClick ( Sender : TObject ) ;
                     procedure ClosePortButtonClick(Sender: TObject);
                     procedure ExitButtonClick ( Sender : TObject ) ;
                     procedure ResetButtonClick(Sender: TObject);
                     procedure SendButtonClick(Sender: TObject);
                     procedure GeneralTimerTimer ( Sender : TObject ) ;
                     procedure EnableTotalButtons ;
                     procedure DisableTotalButtons ;
                     procedure AddToCyclic ( Ch : char ) ;
                     function GetCh ( ) : char ;
  private
    { private declarations }
  public
    { public declarations }
                     CyclicBuffer        : CyclicBufferRecT ;
                     RecvCommand         : string ;
  end;

var
  SerwerCfgForm: TSerwerCfgForm;

implementation

{$R *.lfm}

{ TSerwerCfgForm }

{$IFDEF LINUX}

function IsPortAvailable ( portName : string ) : boolean;
var
  aPort : TBlockSerial;
begin
  aPort := TBlockSerial.Create;
  aPort.Connect( portName );
  IsPortAvailable := aPort.LastError = 0;
  aPort.Free; // spoczko, destruktor robi close
end;

function GetSerialPortNamesEx : string;
var
  serialPorts : TStringList;
  i : integer;
  res : string;
begin
  res := '';
  serialPorts := TStringList.Create;
  try
    FindAllFiles( serialPorts, '/dev', 'ttyS*;ttyUSB*;ttyACM*', false );
    for i := 0 to serialPorts.Count-1 do
    begin
       if IsPortAvailable( serialPorts.Strings[i] ) then
       begin
         if length( res ) <> 0 then res := res + ',';
         res := res + serialPorts.Strings[i];
       end;
    end;
  finally
    serialPorts.Free;
  end;
  GetSerialPortNamesEx := res;
end;
{$ENDIF}

procedure TSerwerCfgForm.FormCreate ( Sender : TObject ) ;
//++tasza
var
  serialPorts     : string ; //tasza
//--tasza
begin (* TSerwerCfgForm.FormCreate *)
  //++tasza
  {$IFNDEF LINUX}
  {$NOTE windowsowe GetSerialPortNames }
  serialPorts := GetSerialPortNames ;
  {$ELSE}
  {$NOTE wlasna implementacja GetSerialPortNamesEx }
  serialPorts := GetSerialPortNamesEx ;
  {$ENDIF}                                                                                                                                              
  if serialPorts = '' then
    with Application do
    begin (* 1 *)
     MessageBox ( 'Brak wolnych portów szeregowych.' , 'oops!' , MB_OK + MB_ICONEXCLAMATION ) ;
     Terminate ;
     exit ;
    end (* 1 *) ;
  SerialPortComboBox . Items . CommaText := serialPorts ;
  //--tasza
  SerialPortComboBox . Text := SerialPortComboBox . Items [ 0 ] ;
  GeneralTimer . Enabled := false ;
  DisableTotalButtons ;
end (* TSerwerCfgForm.FormCreate *) ;


procedure TSerwerCfgForm.ClearButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ClearButtonClick *)
  TextMemo . Clear ;
end (* TSerwerCfgForm.ClearButtonClick *) ;


procedure TSerwerCfgForm.ExitButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ExitButtonClick *)
  Application . Terminate ;
end (* TSerwerCfgForm.ExitButtonClick *) ;


procedure TSerwerCfgForm.OpenPortButtonClick ( Sender : TObject ) ;
var
  OpenFail             : boolean ;
begin (* TSerwerCfgForm.OpenPortButtonClick *)
  OpenFail := false ;
  SerialChannel . Device := SerialPortComboBox . Text ;
  try
    SerialChannel . Open ;
  except
    ShowMessage ( string ( 'Port komunikacyjny ' ) + SerialPortComboBox . Text +
                  string ( ' nie jest dostępny.'  ) ) ;
    OpenFail := true ;
  end (* try *) ;
  if OpenFail then
  begin (* 1 *)
    SerialChannel . Close ;
    exit ;
  end (* 1 *) ;
  SerialChannel . SetDTR ( true ) ;
  SerialChannel . SetRTS ( false ) ;
  TextMemo . Clear ;
  OpenPortButton . Enabled := false ;
  GeneralTimer. Enabled := true ;
  ConnectLED . Color := clGreen ;
  ConnectLED . Active := true ;
  GeneralTimer . Enabled := true ;
  EnableTotalButtons ;
  CyclicBuffer . FrIndex := 0 ;
  CyclicBuffer . ToIndex := 0 ;
  RecvCommand := string ( '' ) ;
  Sleep ( 100 ) ;
  SerialChannel . SetDTR ( false ) ;
end (* TSerwerCfgForm.OpenPortButtonClick *) ;


procedure TSerwerCfgForm.ClosePortButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ClosePortButtonClick *)
  ConnectLED . Active := false ;
  GeneralTimer . Enabled := false ;
  SerialChannel . Close ;
  DisableTotalButtons ;
end (* TSerwerCfgForm.ClosePortButtonClick *) ;


procedure TSerwerCfgForm.ResetButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ResetButtonClick *)
  SerialChannel . SetDTR ( true ) ;
  Sleep ( 100 ) ;
  SerialChannel . SetDTR ( false ) ;
end (* TSerwerCfgForm.ResetButtonClick *) ;


procedure TSerwerCfgForm.SendButtonClick ( Sender : TObject ) ;
var
  CommandToSend          : string ;
begin (* TSerwerCfgForm.SendButtonClick *)
  CommandToSend := CfgDataEdit . Text + string ( chr ( $0D ) ) ;
  SerialChannel . WriteData ( CommandToSend ) ;
end (* TSerwerCfgForm.SendButtonClick *) ;


procedure TSerwerCfgForm.EnableTotalButtons ;
begin (* TSerwerCfgForm.EnableTotalButtons *)
  OpenPortButton . Enabled := false ;
  ClosePortButton . Enabled := true ;
  ExitButton . Enabled := false ;
  SendButton . Enabled := true ;
  ClearButton . Enabled := true ;
end (* TSerwerCfgForm.EnableTotalButtons *) ;


procedure TSerwerCfgForm.DisableTotalButtons ;
begin (* TSerwerCfgForm.DisableTotalButtons *)
  OpenPortButton . Enabled := true ;
  ClosePortButton . Enabled := false ;
  ExitButton . Enabled := true ;
  SendButton . Enabled := false ;
  ClearButton . Enabled := false ;
end (* TSerwerCfgForm.DisableTotalButtons *) ;


procedure TSerwerCfgForm.AddToCyclic ( Ch : char ) ;
begin (* TSerwerCfgForm.AddToCyclic *)
  CyclicBuffer . Buffer [ CyclicBuffer . ToIndex ] := Ch ;
  CyclicBuffer . ToIndex := CyclicBuffer . ToIndex + 1 ;
  if CyclicBuffer . ToIndex >= CyclicBufferSize then
    CyclicBuffer . ToIndex := 0 ;
end (* TSerwerCfgForm.AddToCyclic *) ;


function TSerwerCfgForm.GetCh ( ) : char ;
var
  Ch                      : char ;
begin (* TSerwerCfgForm.GetCh *)
  if CyclicBuffer . FrIndex = CyclicBuffer . ToIndex then
  begin (* 1 *)
    Ch := chr ( 0 ) ;
  end (* 1 *)
  else
  begin (* 1 *)
    Ch := CyclicBuffer . Buffer [ CyclicBuffer . FrIndex ] ;
    CyclicBuffer . FrIndex := CyclicBuffer . FrIndex + 1 ;
    if CyclicBuffer . FrIndex >= CyclicBufferSize then
      CyclicBuffer . FrIndex := 0 ;
  end (* 1 *) ;
  Result := Ch ;
end (* TSerwerCfgForm.GetCh *) ;


procedure TSerwerCfgForm.GeneralTimerTimer ( Sender : TObject ) ;
var
  Command                : string ;
  Loop                   : word ;
  StringSize             : word ;
  Ch                     : char ;
begin (* TSerwerCfgForm.GeneralTimerTimer *)
  GeneralTimer . Enabled := false ;
  if SerialChannel . DataAvailable then
  begin (* 1 *)
    Command := SerialChannel . ReadData ;
    StringSize := length ( Command ) ;
    for Loop := 1 to StringSize do
    begin (* 2 *)
      Ch := Command [ Loop ] ;
      if Ch < ' ' then
      begin (* 3 *)
        if Ch = chr ( $0D ) then
          AddToCyclic ( Ch ) ;
      end (* 3 *)
      else
      begin (* 3 *)
        AddToCyclic ( Ch ) ;
      end (* 3 *) ;
    end (* 2 *) ;
  end (* 1 *) ;
  while CyclicBuffer . FrIndex <> CyclicBuffer . ToIndex do
  begin (* 1 *)
    Ch := GetCh ( ) ;
    if Ch = chr ( $0D ) then
    begin (* 2 *)
      TextMemo . Lines . Add ( RecvCommand ) ;
      RecvCommand := string ( '' ) ;
    end (* 2 *)
    else
    begin (* 2 *)
      RecvCommand := RecvCommand + Ch ;
    end (* 2 *) ;
  end (* 1 *) ;
  GeneralTimer . Enabled := true ;
end (* TSerwerCfgForm.GeneralTimerTimer *) ;

end.

